
; ******************************************************************************************************************************************
; NAME:				WBCtl.asm
;
;
; DEVELOPER:		Xtreme: http://www.xtremesoftware.com
;
;
; DESCRIPTION:		Displays the MS Web Browser COM Control on a Window.
;					The control can be used to make a custom Web Browser
;					or serve as the interface to an Internet related
;					application. Many other possibilities exist...
;
;
; SPECIAL THANKS:	Ernie Murphy: http://here.is/COMinASM	(I couldn't have begun this project without his tutorials and help)
;					Bill T: http://asm.tsx.com				(Renewed my interest in Win32Asm-COM)
;					Japheth: http://www.japheth.de/			(Solved many major bugs herein and helped me on my way) 
;
; ******************************************************************************************************************************************


; ******************************************************************************************************************************************
; MACROS
; ******************************************************************************************************************************************

; Macro to simply interface declarations
STDMETHOD MACRO name,argl:VARARG
	LOCAL	@tmp_a
	LOCAL	@tmp_b
	@tmp_a	TYPEDEF PROTO argl
	@tmp_b	TYPEDEF PTR @tmp_a
	name	@tmp_b ?
ENDM

; ******************************************************************************************************************************************
; COM INTERFACE FUNCTION PROTOTYPES
; ******************************************************************************************************************************************

; IUnknown
	QueryInterface								   proto :DWORD, :DWORD, :DWORD
	AddRef										   proto :DWORD
	Release										   proto :DWORD

; IOleWindow
	IOleWindow@GetWindow						   proto :DWORD, :DWORD
	IOleWindow@ContextSensitiveHelp				   proto :DWORD, :DWORD

; IOleClientSite methods
	IOleClientSite@SaveObject					   proto :DWORD
	IOleClientSite@GetMoniker					   proto :DWORD, :DWORD, :DWORD, :DWORD
	IOleClientSite@GetContainer					   proto :DWORD, :DWORD
	IOleClientSite@ShowObject					   proto :DWORD
	IOleClientSite@OnShowWindow					   proto :DWORD, :DWORD
	IOleClientSite@RequestNewObjectLayout		   proto :DWORD

; IOleInPlaceSite Methods
	IOleInPlaceSite@CanInPlaceActivate			   proto :DWORD
	IOleInPlaceSite@OnInPlaceActivate			   proto :DWORD
	IOleInPlaceSite@OnUIActivate				   proto :DWORD
	IOleInPlaceSite@GetWindowContext			   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD
	IOleInPlaceSite@Scroll						   proto :DWORD, :SIZEL
	IOleInPlaceSite@OnUIDeactivate				   proto :DWORD, :DWORD
	IOleInPlaceSite@OnInPlaceDeactivate			   proto :DWORD
	IOleInPlaceSite@DiscardUndoState			   proto :DWORD
	IOleInPlaceSite@DeactivateAndUndo			   proto :DWORD
	IOleInPlaceSite@OnPosRectChange				   proto :DWORD, :DWORD 

; IDispatch Methods
	IDispatch@GetTypeInfoCount					   proto :DWORD, :DWORD 
	IDispatch@GetTypeInfo						   proto :DWORD, :DWORD, :DWORD, :DWORD
	IDispatch@GetIDsOfNames						   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD
	IDispatch@Invoke_							   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD

; IDocHostUIHandler Methods
	IDocHostUIHandler@ShowContextMenu			   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@GetHostInfo				   proto :DWORD, :DWORD
	IDocHostUIHandler@ShowUI					   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@HideUI					   proto :DWORD
	IDocHostUIHandler@UpdateUI					   proto :DWORD
	IDocHostUIHandler@EnableModeless			   proto :DWORD, :DWORD
	IDocHostUIHandler@OnDocWindowActivate		   proto :DWORD, :DWORD
	IDocHostUIHandler@OnFrameWindowActivate		   proto :DWORD, :DWORD
	IDocHostUIHandler@ResizeBorder				   proto :DWORD, :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@TranslateAccelerator		   proto :DWORD, :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@GetOptionKeyPath			   proto :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@GetDropTarget				   proto :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@GetExternal				   proto :DWORD, :DWORD
	IDocHostUIHandler@TranslateUrl				   proto :DWORD, :DWORD, :DWORD, :DWORD
	IDocHostUIHandler@FilterDataObject			   proto :DWORD, :DWORD, :DWORD

; IDocHostShowUI Methods
	IDocHostShowUI@ShowHelp						   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :POINT, :DWORD
	IDocHostShowUI@ShowMessage					   proto :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD, :DWORD

; Misc. Methods
	CreateObject								   proto :DWORD
	GetConnectionPoint							   proto :DWORD, :DWORD, :DWORD, :DWORD
	AdviseWBEventSink							   proto :DWORD
	CreateWebBrowser							   proto :DWORD
	CleanupWebBrowser							   proto :DWORD
	UnadviseWBEventSink							   proto :DWORD
	ResizeWebBrowser							   proto :DWORD
	
; ******************************************************************************************************************************************
; CLIENT SITE INTEFACE DEFINITIONS AND PROTOTYPES
; ******************************************************************************************************************************************

IIUnknownVtbl struct
	STDMETHOD	QueryInterface, pif:DWORD, riid:DWORD, ppv:DWORD
	STDMETHOD	AddRef, pif:DWORD
	STDMETHOD	Release, pif:DWORD
IIUnknownVtbl ends

IIOleWindowVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	GetWindow,pif:DWORD,phwnd:DWORD
	STDMETHOD	ContextSensitiveHelp,pif:DWORD,fEnterMode:DWORD
IIOleWindowVtbl ends

IIOleClientSiteVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	SaveObject,pif:DWORD
	STDMETHOD	GetMoniker,pif:DWORD,dwAssign:DWORD,dwWhichMoniker:DWORD,ppmk:DWORD
	STDMETHOD	GetContainer,pif:DWORD,ppContainer:DWORD
	STDMETHOD	ShowObject,pif:DWORD
	STDMETHOD	OnShowWindow,pif:DWORD,fShow:DWORD
	STDMETHOD	RequestNewObjectLayout,pif:DWORD
IIOleClientSiteVtbl ends

IIOleInPlaceSiteVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	GetWindow,pif:DWORD,phwnd:DWORD
	STDMETHOD	ContextSensitiveHelp,pif:DWORD,fEnterMode:DWORD
	STDMETHOD	CanInPlaceActivate,pif:DWORD
	STDMETHOD	OnInPlaceActivate,pif:DWORD
	STDMETHOD	OnUIActivate,pif:DWORD
	STDMETHOD	GetWindowContext,pif:DWORD,ppFrame:DWORD,ppDoc:DWORD,lprcPosRect:DWORD,lprcClipRect:DWORD,lpFrameInfo:DWORD
	STDMETHOD	Scroll,pif:DWORD,scrollExtent:SIZEL
	STDMETHOD	OnUIDeactivate,pif:DWORD,fUndoable:DWORD
	STDMETHOD	OnInPlaceDeactivate,pif:DWORD
	STDMETHOD	DiscardUndoState,pif:DWORD
	STDMETHOD	DeactivateAndUndo,pif:DWORD
	STDMETHOD	OnPosRectChange,pif:DWORD,lprcPosRect:DWORD 
IIOleInPlaceSiteVtbl ends

IIDispatchVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	GetTypeInfoCount,pif:DWORD,pctinfo:DWORD
	STDMETHOD	GetTypeInfo,pif:DWORD,iTInfo:DWORD,lcid:DWORD,ppTInfo:DWORD
	STDMETHOD	GetIDsOfNames,pif:DWORD,riid:DWORD,rgszNames:DWORD,cNames:DWORD,lcid:DWORD,rgDispId:DWORD
	STDMETHOD	Invoke_,pif:DWORD,dispIdMember:DWORD,riid:DWORD,lcid:DWORD,wFlags:DWORD,pDispParams:DWORD,pVarResult:DWORD,pExcepInfo:DWORD,puArgErr:DWORD
IIDispatchVtbl ends

IIDocHostUIHandlerVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	ShowContextMenu,pif:DWORD,dwID:DWORD,ppt:DWORD,pcmdtReserved:DWORD,pdispReserved:DWORD
	STDMETHOD	GetHostInfo,pif:DWORD,pInfo:DWORD
	STDMETHOD	ShowUI,pif:DWORD,dwID:DWORD,pActiveObject:DWORD,pCommandTarget:DWORD,pFrame:DWORD,pDoc:DWORD
	STDMETHOD	HideUI,pif:DWORD
	STDMETHOD	UpdateUI,pif:DWORD
	STDMETHOD	EnableModeless,pif:DWORD,fEnable:DWORD
	STDMETHOD	OnDocWindowActivate,pif:DWORD,fActivate:DWORD
	STDMETHOD	OnFrameWindowActivate,pif:DWORD,fActivate:DWORD
	STDMETHOD	ResizeBorder,pif:DWORD,prcBorder:DWORD,pUIWindow:DWORD,fRameWindow:DWORD
	STDMETHOD	TranslateAccelerator,pif:DWORD,lpMsg:DWORD,pguidCmdGroup:DWORD,nCmdID:DWORD
	STDMETHOD	GetOptionKeyPath,pif:DWORD,pchKey:DWORD,dw_:DWORD
	STDMETHOD	GetDropTarget,pif:DWORD,pDropTarget:DWORD,ppDropTarget:DWORD
	STDMETHOD	GetExternal,pif:DWORD,ppDispatch:DWORD
	STDMETHOD	TranslateUrl,pif:DWORD,dwTranslate:DWORD,pchURLIn:DWORD,ppchURLOut:DWORD
	STDMETHOD	FilterDataObject,pif:DWORD,pDO:DWORD,ppDORet:DWORD
IIDocHostUIHandlerVtbl ends

IIDocHostShowUIVtbl struct
	STDMETHOD	QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	AddRef,pif:DWORD
	STDMETHOD	Release,pif:DWORD
	STDMETHOD	ShowHelp,pif:DWORD,hwnd:DWORD,pszHelpFile:DWORD,uCommand:DWORD,dwData:DWORD,ptMouse:POINT,pDispatchObjectHit:DWORD
	STDMETHOD	ShowMessage,pif:DWORD,hwnd:DWORD,lpstrText:DWORD,lpstrCaption:DWORD,dwType:DWORD,lpstrHelpFile:DWORD,dwHelpContext:DWORD,plResult:DWORD
IIDocHostShowUIVtbl ends

; ******************************************************************************************************************************************
; COM OBJECT DEPENDANT INTERFACE PROTOTYPES
; ******************************************************************************************************************************************

IWebBrowser struct
	STDMETHOD	IWebBrowser_QueryInterface,pif:DWORD,riid:DWORD,ppv:DWORD
	STDMETHOD	IWebBrowser_AddRef,pif:DWORD
	STDMETHOD	IWebBrowser_Release,pif:DWORD
	STDMETHOD	IWebBrowser_GetTypeInfoCount,pif:DWORD,pctinfo:DWORD
	STDMETHOD	IWebBrowser_GetTypeInfo,pif:DWORD,iTInfo:DWORD,lcid:DWORD,ppTInfo:DWORD
	STDMETHOD	IWebBrowser_GetIDsOfNames,pif:DWORD,riid:DWORD,rgszNames:DWORD,cNames:DWORD,lcid:DWORD,rgDispId:DWORD
	STDMETHOD	IWebBrowser_Invoke_,pif:DWORD,dispIdMember:DWORD,riid:DWORD,lcid:DWORD,wFlags:DWORD,pDispParams:DWORD,pVarResult:DWORD,pExcepInfo:DWORD,puArgErr:DWORD
	STDMETHOD	IWebBrowser_GoBack,pif:DWORD
	STDMETHOD	IWebBrowser_GoForward,pif:DWORD
	STDMETHOD	IWebBrowser_GoHome,pif:DWORD
	STDMETHOD	IWebBrowser_GoSearch,pif:DWORD
	STDMETHOD	IWebBrowser_Navigate,pif:DWORD,bstrURL:dword,Flags:ptr,TargetFrameName:ptr,PostData:ptr,Headers:ptr
IWebBrowser ends

; ******************************************************************************************************************************************
; REQUIRED STRUCT DEFINITIONS
; ******************************************************************************************************************************************

Object struct
	iUnknown			dd ?
	iBase0				dd ?
	iOleWindow			dd ?
	iBase1				dd ?
	iOleClientSite		dd ?
	iBase2				dd ?
	iOleInPlaceSite		dd ?
	iBase3				dd ?
	iDispatch			dd ?
	iBase4				dd ?
	iDocHostUIHandler	dd ?
	iBase5				dd ?
	iDocHostShowUI		dd ?
	iBase6				dd ?
	_pWebBrowserOC		dd ?
	_pIOleIPObject		dd ?
	_pIOleObject		dd ?
	_dwWBCookie			dd ?
	nRefCount			dd ?
	hMem				dd ?
	m_bInPlaceActive	db ?
Object ends


DOCHOSTUIINFO struct
	cbSize				dd ?
	dwFlags				dd ?
	dwDoubleClick		dd ?
;	pchHostCss			dd ?
;	pchHostNS			dd ?
DOCHOSTUIINFO ends


;DISPPARAMS struct
;	VARIANTARG FAR* rgvarg
;	DISPID FAR* rgdispidNamedArgs
;	cArgs				dd ? 
;	cNamedArgs			dd ?
;DISPPARAMS ends

.data

; ******************************************************************************************************************************************
; CLIENT SITE VTABLE STRUCTS 
; ******************************************************************************************************************************************

vtblIUnknown label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release


vtblIOleWindow label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IOleWindow@GetWindow
		dd		offset IOleWindow@ContextSensitiveHelp


vtblIOleClientSite label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IOleClientSite@SaveObject
		dd		offset IOleClientSite@GetMoniker
		dd		offset IOleClientSite@GetContainer
		dd		offset IOleClientSite@ShowObject
		dd		offset IOleClientSite@OnShowWindow
		dd		offset IOleClientSite@RequestNewObjectLayout


vtblIOleInPlaceSite label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IOleWindow@GetWindow
		dd		offset IOleWindow@ContextSensitiveHelp
		dd		offset IOleInPlaceSite@CanInPlaceActivate
		dd		offset IOleInPlaceSite@OnInPlaceActivate
		dd		offset IOleInPlaceSite@OnUIActivate
		dd		offset IOleInPlaceSite@GetWindowContext
		dd		offset IOleInPlaceSite@Scroll
		dd		offset IOleInPlaceSite@OnUIDeactivate
		dd		offset IOleInPlaceSite@OnInPlaceDeactivate
		dd		offset IOleInPlaceSite@DiscardUndoState
		dd		offset IOleInPlaceSite@DeactivateAndUndo
		dd		offset IOleInPlaceSite@OnPosRectChange


vtblIDispatch label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IDispatch@GetTypeInfoCount
		dd		offset IDispatch@GetTypeInfo
		dd		offset IDispatch@GetIDsOfNames
		dd		offset IDispatch@Invoke_


vtblIDocHostUIHandler label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IDocHostUIHandler@ShowContextMenu
		dd		offset IDocHostUIHandler@GetHostInfo
		dd		offset IDocHostUIHandler@ShowUI
		dd		offset IDocHostUIHandler@HideUI
		dd		offset IDocHostUIHandler@UpdateUI
		dd		offset IDocHostUIHandler@EnableModeless
		dd		offset IDocHostUIHandler@OnDocWindowActivate
		dd		offset IDocHostUIHandler@OnFrameWindowActivate
		dd		offset IDocHostUIHandler@ResizeBorder
		dd		offset IDocHostUIHandler@TranslateAccelerator
		dd		offset IDocHostUIHandler@GetOptionKeyPath
		dd		offset IDocHostUIHandler@GetDropTarget
		dd		offset IDocHostUIHandler@GetExternal
		dd		offset IDocHostUIHandler@TranslateUrl
		dd		offset IDocHostUIHandler@FilterDataObject


vtblIDocHostShowUI label dword
		dd		offset QueryInterface
		dd		offset AddRef
		dd		offset Release
		dd		offset IDocHostShowUI@ShowHelp
		dd		offset IDocHostShowUI@ShowMessage

; ******************************************************************************************************************************************
; CLASS AND INTERFACE IDs
; ******************************************************************************************************************************************

CLSID_WebBrowser				GUID	<08856F961h, 0340Ah, 011D0h, <0A9h, 06Bh, 000h, 0C0h, 04Fh, 0D7h, 005h, 0A2h>>
IID_IUnknown					GUID	<000000000H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IDispatch					GUID	<000020400H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IOleObject					GUID	<000000112H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IOleClientSite				GUID	<000000118H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IOleInPlaceSite				GUID	<000000119H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IDocHostUIHandler			GUID	<0bd3f23c0H, 0d43eH, 011cfH, <089H, 03bH, 000H, 0aaH, 000H, 0bdH, 0ceH, 01aH>>
IID_IDocHostShowUI				GUID	<0c4d244b0H, 0d43eH, 011cfH, <089H, 03bH, 000H, 0aaH, 000H, 0bdH, 0ceH, 01aH>>
DIID_DWebBrowserEvents2			GUID	<034A715A0H, 06587H, 011D0H, <092H, 04AH, 000H, 020H, 0AFH, 0C7H, 0ACH, 04DH>>
IID_IWebBrowser					GUID	<0EAB22AC1H, 030C1H, 011CFH, <0A7H, 0EBH, 000H, 000H, 0C0H, 05BH, 0AEH, 00BH>>
IID_IOleInPlaceObject			GUID	<000000113H, 00000H, 00000H, <0C0H, 000H, 000H, 000H, 000H, 000H, 000H, 046H>>
IID_IConnectionPointContainer	GUID	<0B196B284H, 0BAB4H, 0101AH, <0B6H, 09CH, 000H, 0AAH, 000H, 034H, 01DH, 007H>>

; ******************************************************************************************************************************************
; REQUIRED EQUATES
; ******************************************************************************************************************************************

DOCHOSTUIFLAG_DIALOG					equ 1
DOCHOSTUIFLAG_DISABLE_HELP_MENU			equ 2
DOCHOSTUIFLAG_NO3DBORDER				equ 4
DOCHOSTUIFLAG_SCROLL_NO					equ 8
DOCHOSTUIFLAG_DISABLE_SCRIPT_INACTIVE	equ 16
DOCHOSTUIFLAG_OPENNEWWIN				equ 32
DOCHOSTUIFLAG_DISABLE_OFFSCREEN			equ 64
DOCHOSTUIFLAG_FLAT_SCROLLBAR			equ 128
DOCHOSTUIFLAG_DIV_BLOCKDEFAULT			equ 256
DOCHOSTUIFLAG_ACTIVATE_CLIENTHIT_ONLY	equ 512

DOCHOSTUIDBLCLK_DEFAULT					equ 0
DOCHOSTUIDBLCLK_SHOWPROPERTIES			equ 1
DOCHOSTUIDBLCLK_SHOWCODE				equ 2

OLEIVERB_INPLACEACTIVATE				equ 0FFFFFFFBh

DISPID_BEFORENAVIGATE					equ 064h				; This is sent before navigation to give a chance to abort
DISPID_NAVIGATECOMPLETE					equ 065h				; In async, this is sent when we have enough to show
DISPID_STATUSTEXTCHANGE					equ 066h
DISPID_QUIT								equ 067h
DISPID_DOWNLOADCOMPLETE					equ 068h
DISPID_COMMANDSTATECHANGE				equ 069h
DISPID_DOWNLOADBEGIN					equ 06Ah
DISPID_NEWWINDOW						equ 06Bh				; sent when a new window should be created
DISPID_PROGRESSCHANGE					equ 06Ch				; sent when download progress is updated
DISPID_WINDOWMOVE						equ 06Dh				; sent when main window has been moved
DISPID_WINDOWRESIZE						equ 06Eh				; sent when main window has been sized
DISPID_WINDOWACTIVATE					equ 06Fh				; sent when main window has been activated
DISPID_PROPERTYCHANGE					equ 070h				; sent when the PutProperty method is called
DISPID_TITLECHANGE						equ 071h				; sent when the document title changes
DISPID_TITLEICONCHANGE					equ 072h				; sent when the top level window icon may have changed.
DISPID_FRAMEBEFORENAVIGATE				equ 0C8h
DISPID_FRAMENAVIGATECOMPLETE			equ 0C9h
DISPID_FRAMENEWWINDOW					equ 0CCh
DISPID_BEFORENAVIGATE2					equ 0FAh				; hyperlink clicked on
DISPID_NEWWINDOW2						equ 0FBh
DISPID_NAVIGATECOMPLETE2				equ 0FCh				; UIActivate new document
DISPID_ONQUIT							equ 0FDh
DISPID_ONVISIBLE						equ 0FEh				; sent when the window goes visible/hidden
DISPID_ONTOOLBAR						equ 0FFh				; sent when the toolbar should be shown/hidden
DISPID_ONMENUBAR						equ 100h				; sent when the menubar should be shown/hidden
DISPID_ONSTATUSBAR						equ 101h				; sent when the statusbar should be shown/hidden
DISPID_ONFULLSCREEN						equ 102h				; sent when kiosk mode should be on/off
DISPID_DOCUMENTCOMPLETE					equ 103h				; new document goes ReadyState_Complete
DISPID_ONTHEATERMODE					equ 104h				; sent when theater mode should be on/off
DISPID_ONADDRESSBAR						equ 105h				; sent when the address bar should be shown/hidden

MB_ERROR								equ MB_SETFOREGROUND or MB_ICONSTOP

.code

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

CreateWebBrowser proc uses ebx,hWin:DWORD
	LOCAL	pif:DWORD
	LOCAL	rcClient:RECT
	LOCAL	msg:MSG

	; Create the client site
		invoke CreateObject,addr pif
		.IF_FAILED
				szText szmsg0,"Cannot Allocate Memory"
				invoke MessageBox,0,addr szmsg0,addr szDisplayName,MB_ERROR
				jmp		@fail
		.ENDIF
		invoke GetWindowLong,hWin,0
		mov		ebx,pif
		mov		[ebx].Object.hMem,eax
		mov		[eax].WBDATA.pObj,ebx
	; Create an instance of the WebBrowser Control
		invoke CoCreateInstance,ADDR CLSID_WebBrowser,NULL,CLSCTX_INPROC_SERVER or CLSCTX_INPROC_HANDLER,ADDR IID_IOleObject,ADDR [ebx].Object._pIOleObject
		.IF_FAILED
				szText szmsg1,"Cannot Load Internet Explorer"
				invoke MessageBox,0,addr szmsg1,addr szDisplayName,MB_ERROR
				jmp		@fail
		.ENDIF
	; Set the Web Browser Control's site
		mov		eax,pif
		lea		eax,[eax].Object.iOleClientSite
		coinvoke [ebx].Object._pIOleObject,IOleObject,SetClientSite,eax
		.IF_FAILED
				jmp		@fail
		.ENDIF
	; Inplace Activate the Web Browser Control
		invoke GetClientRect,hWin,addr rcClient
		coinvoke [ebx].Object._pIOleObject,IOleObject,DoVerb,OLEIVERB_INPLACEACTIVATE,addr msg,pif,0,hWin,addr rcClient
		.IF_FAILED
				jmp		@fail
		.ENDIF
	; Query OLE Object interface for pointer to Web Browser
		coinvoke [ebx].Object._pIOleObject,IOleObject,QueryInterface,addr IID_IWebBrowser,addr [ebx].Object._pWebBrowserOC
		.IF_FAILED
				jmp		@fail
		.endif
	; Set up an event sink for the WebBrowser events
		.IF  [ebx].Object._pWebBrowserOC
				invoke AdviseWBEventSink,pif
		.ELSE
				jmp		@fail
		.ENDIF
	; Query the Browser Pointer for the InPlaceObject
		coinvoke [ebx].Object._pWebBrowserOC,IUnknown,QueryInterface,addr IID_IOleInPlaceObject,addr [ebx].Object._pIOleIPObject
		.IF_FAILED
				jmp		@fail
		.endif
	; Exit successfully
		mov		eax,S_OK
		jmp		@ex
	@fail:
		mov eax, E_FAIL
	@ex:
		ret

CreateWebBrowser endp

CleanupWebBrowser proc uses ebx,pif:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	.IF [ebx].Object._pIOleIPObject
		coinvoke [ebx].Object._pIOleIPObject,IOleInPlaceObject,Release
		mov [ebx].Object._pIOleIPObject,NULL
	.ENDIF
	.IF [ebx].Object._pWebBrowserOC
		invoke UnadviseWBEventSink,pif
		coinvoke [ebx].Object._pWebBrowserOC,IWebBrowser,Release
		mov [ebx].Object._pWebBrowserOC,NULL
	.ENDIF
	.IF [ebx].Object._pIOleObject
		coinvoke [ebx].Object._pIOleObject,IOleObject,Release
		mov [ebx].Object._pIOleObject,NULL
	.ENDIF
	mov		eax,[ebx].Object.nRefCount
	.while eax
		invoke Release,ebx
	.endw
	ret

CleanupWebBrowser endp

CreateObject proc pobj:DWORD

	; Set *ppv to 0
	mov		eax,pobj
	mov		dword ptr [eax],0
	; allocate object
	invoke GetProcessHeap
	invoke HeapAlloc,eax,HEAP_ZERO_MEMORY,sizeof Object
	or		eax,eax
	jnz		@1
	; alloc failed, so return
	mov		eax,E_FAIL
	jmp		@cleanup
  @1: 
	; Setup the Site
	mov		edx,eax
	mov		eax,pobj
	mov		[eax],edx
	mov		[edx].Object.nRefCount,0														  ; set this interface's ref count to 0
	mov		[edx].Object.iUnknown,offset vtblIUnknown					   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase0,0
	mov		[edx].Object.iOleWindow,offset vtblIOleWindow				   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase1,8
	mov		[edx].Object.iOleClientSite,offset vtblIOleClientSite		   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase2,16
	mov		[edx].Object.iOleInPlaceSite,offset vtblIOleInPlaceSite	   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase3,24
	mov		[edx].Object.iDispatch,offset vtblIDispatch				   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase4,32
	mov		[edx].Object.iDocHostUIHandler,offset vtblIDocHostUIHandler   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase5,40
	mov		[edx].Object.iDocHostShowUI,offset vtblIDocHostShowUI		   ; save the offset(pointer) to the interface VTABLE in lpVtbl
	mov		[edx].Object.iBase6,48
	mov		eax,S_OK
  @cleanup:
	ret

CreateObject endp

ResizeWebBrowser proc uses ebx,hWin:HWND
	LOCAL	rcClient:RECT
	LOCAL	rct:RECT
	LOCAL	capHt:DWORD

	invoke GetWindowLong,hWin,0
	mov		ebx,eax
	mov		ecx,[ebx].WBDATA.pObj
	.IF [ecx].Object._pWebBrowserOC
		.IF [ecx].Object.m_bInPlaceActive
			.IF [ecx].Object._pIOleIPObject
				push	ecx
				mov		capHt,0
				invoke GetClientRect,[ebx].WBDATA.hWBParentWnd,addr rcClient
				.if [ebx].WBDATA.hWBComboWnd || [ebx].WBDATA.hWBCaptionWnd
					mov		capHt,21
					.if [ebx].WBDATA.hWBComboWnd
						invoke GetClientRect,[ebx].WBDATA.hWBComboWnd,addr rct
						mov		eax,rct.bottom
						sub		eax,rct.top
						mov		capHt,eax
					.endif
					.if [ebx].WBDATA.hWBCaptionWnd
						.if [ebx].WBDATA.hWBComboWnd
							invoke MoveWindow,[ebx].WBDATA.hWBCaptionWnd,0,0,50,capHt,TRUE
							mov		eax,rcClient.right
							sub		eax,50
							invoke MoveWindow,[ebx].WBDATA.hWBComboWnd,50,0,eax,100,TRUE
						.else
							invoke MoveWindow,[ebx].WBDATA.hWBCaptionWnd,0,0,rcClient.right,capHt,TRUE
						.endif
					.else
						invoke MoveWindow,[ebx].WBDATA.hWBComboWnd,0,0,rcClient.right,100,TRUE
					.endif
					mov		eax,capHt
					add		rcClient.top,eax
				.endif
				.if [ebx].WBDATA.hWBStatusWnd
					invoke MoveWindow,[ebx].WBDATA.hWBStatusWnd,0,0,0,0,TRUE
					sub		rcClient.bottom,20
				.endif
				pop		ecx
				coinvoke [ecx].Object._pIOleIPObject, IOleInPlaceObject,SetObjectRects,addr rcClient,addr rcClient
			.ENDIF
		.ENDIF
	.ENDIF
	ret

ResizeWebBrowser endp

AdviseWBEventSink proc uses ebx,pif:DWORD
	LOCAL	pCP:DWORD
	LOCAL	iTemp:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	.IF [ebx].Object._pWebBrowserOC
	   invoke GetConnectionPoint,pif,[ebx].Object._pWebBrowserOC,addr DIID_DWebBrowserEvents2,addr pCP
		.IF_SUCCEEDED
			mov		eax,pif
			lea		eax,[eax].Object.iDispatch
			mov		iTemp,eax
			coinvoke pCP,IConnectionPoint,Advise,iTemp,addr [ebx].Object._dwWBCookie
			coinvoke pCP,IConnectionPoint,Release
		.ENDIF
	.ENDIF
	ret

AdviseWBEventSink endp

UnadviseWBEventSink proc uses ebx,pif:DWORD
	LOCAL	pCP:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	.IF [ebx].Object._pWebBrowserOC
		.IF [ebx].Object._dwWBCookie 
			invoke GetConnectionPoint,ebx,[ebx].Object._pWebBrowserOC,addr DIID_DWebBrowserEvents2,addr pCP
			.IF_SUCCEEDED
				coinvoke pCP,IConnectionPoint,Unadvise,[ebx].Object._dwWBCookie
				coinvoke pCP,IConnectionPoint,Release
				dec		[ebx].Object._dwWBCookie
			.ENDIF
		.ENDIF
	.ENDIF
	ret

UnadviseWBEventSink endp

GetConnectionPoint proc uses ebx,pif:DWORD,pUnk:DWORD,riid:DWORD,ppCP:DWORD
	LOCAL	pCPC:DWORD

	mov		eax,E_FAIL
	mov		ebx,pif
	sub		ebx,[ebx+4]
	.IF [ebx].Object._pWebBrowserOC
		coinvoke pUnk,IUnknown,QueryInterface,addr IID_IConnectionPointContainer,addr pCPC
		.IF_FAILED
			return E_FAIL
		.ENDIF
		coinvoke pCPC,IConnectionPointContainer,FindConnectionPoint,riid,ppCP
		push	eax
		coinvoke pCPC,IConnectionPointContainer,Release
		pop		eax	
	.ENDIF
	ret

GetConnectionPoint endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

QueryInterface proc pif:DWORD,riid:DWORD,ppv:DWORD
	LOCAL	iface:DWORD

	invoke IsEqualGUID,riid,addr IID_IUnknown
	test	eax,eax
	mov		iface,1
	jnz		@F
	invoke IsEqualGUID,riid,addr IID_IOleClientSite
	test	eax,eax
	mov		iface,2
	jnz		@F
	invoke IsEqualGUID,riid,addr IID_IDocHostUIHandler
	test	eax,eax
	mov		iface,3
	jnz		@F
	invoke IsEqualGUID,riid,addr IID_IDocHostShowUI
	test	eax,eax
	mov		iface,4
	jnz		@F
	invoke IsEqualGUID,riid,addr IID_IOleInPlaceSite
	test	eax,eax
	mov		iface,5
	jnz		@F
	invoke IsEqualGUID,riid,addr IID_IDispatch
	test	eax,eax
	mov		iface,6
	jnz		@F
	invoke IsEqualGUID,riid,addr DIID_DWebBrowserEvents2
	test	eax,eax
	mov		iface,6
	jnz		@F
	jmp		@Error
  @@:
	mov		eax,pif											; move the contents at PIF into eax
	sub		eax,[eax + 4]									; 
	.IF iface==1
		mov		ecx,(Object ptr [eax]).iUnknown				; Move iUnknown's lpVtbl into ecx
		lea		eax,(Object ptr [eax]).iUnknown				; Move the offset of "Object.iUnknown" to eax.
		mov		edx,[ppv]									; Get the pointer at address PPV
		mov		[edx],eax									; Make the pointer point to iUnknown 
		invoke (IIUnknownVtbl ptr [ecx]).AddRef,pif		; 
	.ELSEIF iface==2
		mov		ecx,(Object ptr [eax]).iOleClientSite
		lea		eax,(Object ptr [eax]).iOleClientSite
		mov		edx,[ppv]
		mov		[edx], eax
		invoke (IIOleClientSiteVtbl ptr [ecx]).AddRef,pif
	.ELSEIF iface==3
		mov		ecx,(Object ptr [eax]).iDocHostUIHandler
		lea		eax,(Object ptr [eax]).iDocHostUIHandler
		mov		edx,[ppv]
		mov		[edx], eax
		invoke (IIDocHostUIHandlerVtbl ptr [ecx]).AddRef,pif
	.ELSEIF iface==4
		mov		ecx,(Object ptr [eax]).iDocHostShowUI
		lea		eax,(Object ptr [eax]).iDocHostShowUI
		mov		edx,[ppv]
		mov		[edx],eax
		invoke (IIDocHostShowUIVtbl ptr [ecx]).AddRef,pif
	.ELSEIF iface==5
		mov		ecx,(Object ptr [eax]).iOleInPlaceSite
		lea		eax,(Object ptr [eax]).iOleInPlaceSite
		mov		edx,[ppv]
		mov		[edx], eax
		invoke (IIOleInPlaceSiteVtbl ptr [ecx]).AddRef,pif
	.ELSEIF iface==6
		mov		ecx,(Object ptr [eax]).iDispatch
		lea		eax,(Object ptr [eax]).iDispatch
		mov		edx,[ppv]
		mov		[edx],eax
		invoke (IIDispatchVtbl ptr [ecx]).AddRef,pif
	.ENDIF
	mov		eax,S_OK
	jmp		@ex
  @Error:
	mov		eax,ppv
	mov		dword ptr [eax],0
	mov		eax,E_NOINTERFACE
  @ex:
	ret

QueryInterface endp

AddRef proc uses ebx,pif:DWORD

	; Inc the ref count
	mov		ebx,pif
	sub		ebx,[ebx+4]
	invoke InterlockedIncrement,addr [ebx].Object.nRefCount
	ret
		
AddRef endp

Release proc uses ebx,pif:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	invoke InterlockedDecrement,addr [ebx].Object.nRefCount
	.IF eax==0
		; NEVER CALLED: WHY???
		mov		eax,pif
		sub		eax,[eax+4]
		push	eax
		push	0
		invoke GetProcessHeap
		push	eax
		call	HeapFree
		xor		eax,eax
	.ENDIF
	ret

Release endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IOleWindow@GetWindow proc uses ebx,pif:DWORD,phwnd:DWORD

	; *phwnd = hWnd (hWnd is the Main Window)
		mov		ebx,pif
		sub		ebx,[ebx+4]
		mov		ebx,[ebx].Object.hMem
		mov		eax,phwnd
		mov		edx,[ebx].WBDATA.hWBParentWnd
		mov		[eax],edx
	; return S_OK
		mov		eax,S_OK
		ret

IOleWindow@GetWindow endp

IOleWindow@ContextSensitiveHelp proc pif:DWORD,fEnterMode:DWORD

	return E_NOTIMPL

IOleWindow@ContextSensitiveHelp endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IOleClientSite@SaveObject proc pif:DWORD

	return E_NOTIMPL

IOleClientSite@SaveObject endp

IOleClientSite@GetMoniker proc pif:DWORD,dwAssign:DWORD,dwWhichMoniker:DWORD,ppmk:DWORD

	return E_NOTIMPL

IOleClientSite@GetMoniker endp

IOleClientSite@GetContainer proc pif:DWORD,ppContainer:DWORD

	mov		eax,ppContainer
	mov		dword ptr [eax],0
	return E_NOINTERFACE

IOleClientSite@GetContainer endp

IOleClientSite@ShowObject proc pif:DWORD

	return S_OK

IOleClientSite@ShowObject endp

IOleClientSite@OnShowWindow proc pif:DWORD,fShow:DWORD

	return S_OK

IOleClientSite@OnShowWindow endp

IOleClientSite@RequestNewObjectLayout proc pif:DWORD

	return E_NOTIMPL

IOleClientSite@RequestNewObjectLayout endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IOleInPlaceSite@CanInPlaceActivate proc pif:DWORD

	return S_OK

IOleInPlaceSite@CanInPlaceActivate endp

IOleInPlaceSite@OnInPlaceActivate proc uses ebx,pif:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	mov		[ebx].Object.m_bInPlaceActive, TRUE
	mov		eax,S_OK
	ret

IOleInPlaceSite@OnInPlaceActivate endp

IOleInPlaceSite@OnUIActivate proc pif:DWORD

	return S_OK

IOleInPlaceSite@OnUIActivate endp

IOleInPlaceSite@GetWindowContext proc uses ebx,pif:DWORD,ppFrame:DWORD,ppDoc:DWORD,lprcPosRect:DWORD,lprcClipRect:DWORD,lpFrameInfo:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	mov		ebx,[ebx].Object.hMem
	invoke GetClientRect,[ebx].WBDATA.hWBParentWnd,lprcPosRect
	invoke GetClientRect,[ebx].WBDATA.hWBParentWnd,lprcClipRect
	mov		eax,S_OK
	ret

IOleInPlaceSite@GetWindowContext endp

IOleInPlaceSite@Scroll proc pif:DWORD,scrollExtent:SIZEL

	return E_NOTIMPL

IOleInPlaceSite@Scroll endp

IOleInPlaceSite@OnUIDeactivate proc pif:DWORD,fUndoable:DWORD

	return E_NOTIMPL

IOleInPlaceSite@OnUIDeactivate endp

IOleInPlaceSite@OnInPlaceDeactivate proc uses ebx,pif:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	mov		[ebx].Object.m_bInPlaceActive,FALSE
	.IF [ebx].Object._pIOleIPObject
		coinvoke [ebx].Object._pIOleIPObject,IOleInPlaceObject,Release
		mov [ebx].Object._pIOleIPObject,NULL
	.ENDIF
	return S_OK

IOleInPlaceSite@OnInPlaceDeactivate endp

IOleInPlaceSite@DiscardUndoState proc pif:DWORD

	return E_NOTIMPL

IOleInPlaceSite@DiscardUndoState endp

IOleInPlaceSite@DeactivateAndUndo proc pif:DWORD

	return E_NOTIMPL

IOleInPlaceSite@DeactivateAndUndo endp

IOleInPlaceSite@OnPosRectChange proc pif:DWORD,lprcPosRect:DWORD

	return E_NOTIMPL


IOleInPlaceSite@OnPosRectChange endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IDispatch@GetTypeInfoCount proc pif:DWORD,pctinfo:DWORD

	return E_NOTIMPL

IDispatch@GetTypeInfoCount endp

IDispatch@GetTypeInfo proc pif:DWORD,iTInfo:DWORD,lcid:DWORD,ppTInfo:DWORD

	return E_NOTIMPL

IDispatch@GetTypeInfo endp

IDispatch@GetIDsOfNames proc pif:DWORD,riid:DWORD,rgszNames:DWORD,cNames:DWORD,lcid:DWORD,rgDispId:DWORD

	return DISP_E_UNKNOWNINTERFACE

IDispatch@GetIDsOfNames endp

IDispatch@Invoke_ proc uses ebx,pif:DWORD,dispIdMember:DWORD,riid:DWORD,lcid:DWORD,wFlags:DWORD,pDispParams:DWORD,pVarResult:DWORD,pExcepInfo:DWORD,puArgErr:DWORD

	;.IF dispIdMember == DISPID_STATUSTEXTCHANGE
	;.ELSEIF dispIdMember == DISPID_NAVIGATECOMPLETE
	;.ELSEIF dispIdMember == DISPID_BEFORENAVIGATE
	;.ELSEIF dispIdMember == DISPID_QUIT
	;.ELSEIF dispIdMember == DISPID_DOWNLOADCOMPLETE
	;.ELSEIF dispIdMember == DISPID_COMMANDSTATECHANGE
	;.ELSEIF dispIdMember == DISPID_DOWNLOADBEGIN
	;.ELSEIF dispIdMember == DISPID_NEWWINDOW
	;.ELSEIF dispIdMember == DISPID_PROGRESSCHANGE
	;.ELSEIF dispIdMember == DISPID_WINDOWMOVE
	;.ELSEIF dispIdMember == DISPID_WINDOWRESIZE
	;.ELSEIF dispIdMember == DISPID_WINDOWACTIVATE
	;.ELSEIF dispIdMember == DISPID_PROPERTYCHANGE
	;.ELSEIF dispIdMember == DISPID_TITLECHANGE
	; ;OLE2T(pDispParams->rgvarg[0].bstrVal)
	; ;WideCharToMultiByte( CP_ACP, 0, ui->usri2_full_name, -1, dest, 256, NULL, NULL ); 
	;.ELSEIF dispIdMember == DISPID_TITLEICONCHANGE
	;.ELSEIF dispIdMember == DISPID_FRAMEBEFORENAVIGATE
	;.ELSEIF dispIdMember == DISPID_FRAMENAVIGATECOMPLETE
	;.ELSEIF dispIdMember == DISPID_FRAMENEWWINDOW
	;.ELSEIF dispIdMember == DISPID_BEFORENAVIGATE2
	;.ELSEIF dispIdMember == DISPID_NEWWINDOW2
	;.ELSEIF dispIdMember == DISPID_NAVIGATECOMPLETE2
	;.ELSEIF dispIdMember == DISPID_ONQUIT
	;.ELSEIF dispIdMember == DISPID_ONVISIBLE
	;.ELSEIF dispIdMember == DISPID_ONTOOLBAR
	;.ELSEIF dispIdMember == DISPID_ONMENUBAR
	;.ELSEIF dispIdMember == DISPID_ONSTATUSBAR
	;.ELSEIF dispIdMember == DISPID_ONFULLSCREEN
	;.ELSEIF dispIdMember == DISPID_DOCUMENTCOMPLETE
	;.ELSEIF dispIdMember == DISPID_ONTHEATERMODE
	;.ELSEIF dispIdMember == DISPID_ONADDRESSBAR
	;.ENDIF
	mov		eax,E_NOTIMPL			  
	ret

IDispatch@Invoke_ endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IDocHostUIHandler@ShowContextMenu proc pif:DWORD,dwID:DWORD,ppt:DWORD,pcmdtReserved:DWORD,pdispReserved:DWORD

	return E_FAIL

IDocHostUIHandler@ShowContextMenu endp

IDocHostUIHandler@GetHostInfo proc pif:DWORD,pInfo:DWORD

	mov		eax,pInfo
	assume eax:ptr DOCHOSTUIINFO
	mov		[eax].cbSize,sizeof DOCHOSTUIINFO
	mov		[eax].dwFlags,DOCHOSTUIFLAG_DIALOG or DOCHOSTUIFLAG_DISABLE_HELP_MENU or DOCHOSTUIFLAG_FLAT_SCROLLBAR; or DOCHOSTUIFLAG_NO3DBORDER
	mov		[eax].dwDoubleClick,DOCHOSTUIDBLCLK_SHOWPROPERTIES;DOCHOSTUIDBLCLK_SHOWCODE;DOCHOSTUIDBLCLK_DEFAULT
	assume eax:nothing
	return S_OK

IDocHostUIHandler@GetHostInfo endp

IDocHostUIHandler@ShowUI proc pif:DWORD,dwID:DWORD,pActiveObject:DWORD,pCommandTarget:DWORD,pFrame:DWORD,pDoc:DWORD

	return E_NOTIMPL

IDocHostUIHandler@ShowUI endp

IDocHostUIHandler@HideUI proc pif:DWORD

	return E_NOTIMPL

IDocHostUIHandler@HideUI endp

IDocHostUIHandler@UpdateUI proc pif:DWORD

	return E_NOTIMPL

IDocHostUIHandler@UpdateUI endp

IDocHostUIHandler@EnableModeless proc pif:DWORD,fEnable:DWORD

	return E_NOTIMPL

IDocHostUIHandler@EnableModeless endp

IDocHostUIHandler@OnDocWindowActivate proc pif:DWORD,fActivate:DWORD

	return E_NOTIMPL

IDocHostUIHandler@OnDocWindowActivate endp

IDocHostUIHandler@OnFrameWindowActivate proc pif:DWORD,fActivate:DWORD

	return E_NOTIMPL

IDocHostUIHandler@OnFrameWindowActivate endp

IDocHostUIHandler@ResizeBorder proc pif:DWORD,prcBorder:DWORD,pUIWindow:DWORD,fRameWindow:DWORD

	return E_NOTIMPL

IDocHostUIHandler@ResizeBorder endp

IDocHostUIHandler@TranslateAccelerator proc pif:DWORD,lpMsg:DWORD,pguidCmdGroup:DWORD,nCmdID:DWORD

	return E_NOTIMPL

IDocHostUIHandler@TranslateAccelerator endp

IDocHostUIHandler@GetOptionKeyPath proc pif:DWORD,pchKey:DWORD,dw_:DWORD

	return E_NOTIMPL

IDocHostUIHandler@GetOptionKeyPath endp

IDocHostUIHandler@GetDropTarget proc pif:DWORD,pDropTarget:DWORD,ppDropTarget:DWORD

	return E_NOTIMPL

IDocHostUIHandler@GetDropTarget endp

IDocHostUIHandler@GetExternal proc pif:DWORD,ppDispatch:DWORD

	mov		eax,ppDispatch
	mov		edx,pif
	mov		[eax],edx
	return S_OK

IDocHostUIHandler@GetExternal endp

IDocHostUIHandler@TranslateUrl proc uses ebx,pif:DWORD,dwTranslate:DWORD,pchURLIn:DWORD,ppchURLOut:DWORD

	mov		ebx,pif
	sub		ebx,[ebx+4]
	mov		ebx,[ebx].Object.hMem
	.if [ebx].WBDATA.hWBStatusWnd
		invoke WideCharToMultiByte,CP_ACP,0,pchURLIn,-1,offset buffer,sizeof buffer,0,0
		invoke SetWindowText,[ebx].WBDATA.hWBStatusWnd,offset buffer
	.endif
	return E_NOTIMPL

IDocHostUIHandler@TranslateUrl endp

IDocHostUIHandler@FilterDataObject proc pif:DWORD,pDO:DWORD,ppDORet:DWORD

	return E_NOTIMPL

IDocHostUIHandler@FilterDataObject endp

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

IDocHostShowUI@ShowHelp proc pif:DWORD,hwnd:DWORD,pszHelpFile:DWORD,uCommand:DWORD,dwData:DWORD,ptMouse:POINT,pDispatchObjectHit:DWORD

	return E_NOTIMPL

IDocHostShowUI@ShowHelp endp

IDocHostShowUI@ShowMessage proc pif:DWORD,hwnd:DWORD,lpstrText:DWORD,lpstrCaption:DWORD,dwType:DWORD,lpstrHelpFile:DWORD,dwHelpContext:DWORD,plResult:DWORD

	; This Proc is called by MSHTML when it needs to display a message box.
	.IF plResult
		invoke MessageBox,hwnd,lpstrText,addr szDisplayName,dwType
		invoke MessageBox,hwnd,lpstrText,addr lpstrCaption,dwType
;		mov		edx,plResult
;		mov		edx,[edx]
;		mov		word ptr [edx],S_FALSE
	.ELSE
		invoke MessageBox,hwnd,lpstrText,addr szDisplayName,dwType
	.ENDIF
	;S_OK		Host displayed its UI. MSHTML will not display its message box.
	;S_FALSE	Host did not display its UI. MSHTML will display its message box.
	return S_OK

IDocHostShowUI@ShowMessage endp
